/*
Autor: ukasz Romanowski

Biblioteka standarodowa: <complex>

Opis: Klasa szablonowa complex suy do reprezentacji i manipulacji liczbami zespolonymi.
Operacje zdefiniowane na liczbach zespolonych pozwalaj na swobodne mieszanie obiektw tej
klasy z wbudowanymi typami numerycznymi. Naley zauway, e wikszo operacji wykonywanych
na obiektach tej klasy to zwyke funkcje. Wyjtkami s metody real() i imag() nalece do klasy complex. 
*/

#include <iostream>
#include <complex>
#include <utility>

using namespace std;

typedef complex<double> Complex;

// deklaracja funkcji obliczajacej pierwiastki kwadratowe rwnania o wspczynnikach zespolonych
// implementacja poniej 
pair<Complex, Complex> roots (Complex a, Complex b, Complex c);

int main() {
	// deklaracje wykorzystujce rzne konstruktory dostpne w bibliotece standardowej
   complex<double> cplx_one;               // warto 0 + 0i
   complex<double> cplx_two(3.14);         // warto 3.14 + 0i
   complex<double> cplx_three(1.5, 3.14);   // warto 1.5 + 3.14i
   complex<double> cplx_four(cplx_two);    // warto rwnie 3.14 + 0i

	cout << "cplx_one:   " << cplx_one << endl;
	cout << "cplx_two:   " << cplx_two << endl;
	cout << "cplx_three: " << cplx_three << endl;
	cout << "cplx_four:  " << cplx_four << endl;
   
   // wykorzystanie operatora przypisania oraz konwersji za pomoc konstruktora jednoargumentowego
   cplx_one = cplx_three;                 // staje si 1.5 + 3.14i
   cplx_three = 2.17;                     // staje si 2.17 + 0i

	cout << "cplx_one:   " << cplx_one << endl;
	cout << "cplx_three: " << cplx_three << endl;
   
   // wykorzystanie funkcji polar() do konstrukcji liczby zespolonej na podstawie moduu i kta fazowego
   cplx_four = polar(5.6, 1.8);
   
	cout << "cplx_four:  " << cplx_four << endl;
	
	// wartosc sprzon otrzymujemy za pomoc funkcji conj()
	complex<double> cplx_five = conj(cplx_four);
	
	cout << "cplx_five:  " << cplx_five << endl;
	
	// aby otrzyma wartoci czci rzeczywistej i urojonej korzystamy z funkcji real() i imag()
   cout << cplx_one.real() << "+" << cplx_one.imag() << "i" << endl;
   cout << real(cplx_one)  << "+" << imag(cplx_one)  << "i" << endl;   
   
   // operatory mona wykorzystywa zgodnie z przyjt konwencj
   cout << cplx_one + cplx_two << endl;            // warto (4.64,3.14)
   cout << cplx_one - 3.14 << endl;                // warto (-1.64,3.14)
   cout << 2.75 * cplx_two << endl;                // warto (8.635,0)
   cout << (cplx_one += cplx_three / 2.0) << endl; // warto (2.585,3.14)
   
   // dostepne s rwnie operatory jednoargumentowe + i -
   cout << -cplx_one << endl;                      // warto (-2.585,-3.14)
   
   // do porwnania wartoci dwch liczb zespolonych uywamy operatorw == i !=,
   // pozostae operatory porwnujce s niedostpne
   cout << (cplx_one == cplx_three) << endl;  // warto 0 (fasz)
   cout << (cplx_one != cplx_three) << endl;  // warto 1 (prawda)
   
   // wartoci normalna i modu dostpne s za pomoc funkcji norm() i abs()
   cout << norm(cplx_two) << endl;
   cout << abs(cplx_two)  << endl;
   
   // kt fazowy mona uzyska za pomoc funkcji funkcji arg()
   cout << cplx_four << " w postaci biegunowej wynosi "
        << abs(cplx_four) << "e^" << arg(cplx_four) << "i"<< endl;
   
   /*
   przykad wykorzystania klasy complex<> do obliczenia pierwiastkw rwnania danego wzorem:
      (2,1)*x^2 + (0,1)*x + (5,0) = 0;
   W celu przetestowania dziaania funkcji naley zmienia wartoci a, b i c dane poniej
   */
   
   Complex a(2,1);
   Complex b(0,1);
   Complex c(5,0);
   
   pair<Complex,Complex> res(roots(a,b,c));
   
   cout << "Wyniki przykadowej funkcji roots() obliczajcej pierwiastki rwnania" << endl
        << a << "*x^2 + " << b << "*x + " << c << " = 0" << endl;
   cout << "  x1= " << res.first << endl;
   cout << "  x2= " << res.second << endl;
   
}

// definicja funkcji obliczajacej pierwiastki kwadratowe rwnania o wspczynnikach zespolonych
pair<Complex, Complex> roots (Complex a, Complex b, Complex c)
{
   Complex root = sqrt(b * b - 4.0 * a * c);
   a *= 2.0;
   return make_pair(
      (-b + root)/a, 
      (-b - root)/a
   );
}

